/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.lidar.amapvox.visualization.controllers;

import fr.amap.lidar.amapvox.visualization.engine3d.util.VoxelFilter;
import fr.amap.lidar.amapvox.visualization.views.VisualizationScene;
import net.jcip.annotations.GuardedBy;
import net.jcip.annotations.ThreadSafe;

/**
 * Controller of the properties view.
 */
@ThreadSafe
public class PropertiesController {
    
    /**
     * The current scene.
     */
    @GuardedBy("this")
    private VisualizationScene scene;

    /**
     * Create a property controller with an empty scene.
     */
    public PropertiesController() {
        this.scene = null;
    }
    
    /**
     * Change the current scene.
     * @param scene 
     */
    public final synchronized void setScene(final VisualizationScene scene) {
        this.scene = scene;
    }

    /**
     * Get the current scene.
     * @return 
     */
    public final synchronized VisualizationScene getScene() {
        return scene;
    }
    
    /**
     * Change the displayed property of the voxel space.
     * @param property 
     */
    public final synchronized void changeCurrentProperty(final String property) {
        if (scene != null) {
            scene.getVoxelSpaceComponent().setProperty(property);
            scene.setSomethingsModified();
        }
    }
    
    /**
     * Change the min and max of the current property values.
     * (not used by the UI)
     * @param min
     * @param max 
     */
    public final synchronized void changeMinAndMax(final float min, final float max) {
        if (scene != null) {
            scene.getVoxelSpaceComponent().setColorMinAndMax(min, max);
            scene.setSomethingsModified();
        }
    }
    
    /**
     * Add or replace a particular filter of a voxel property.
     * @param filter 
     */
    public final synchronized void changeFilter(final VoxelFilter filter) {
        if (scene != null && filter != null) {
            scene.getVoxelSpaceComponent().setFilter(filter);
            scene.setSomethingsModified();
        }
    }
    
    /**
     * Remove a filter of a voxel property.
     * @param filter 
     */
    public final synchronized void removeFilter(final int filter) {
        if (scene != null) {
            scene.getVoxelSpaceComponent().removeFilter(filter);
            scene.setSomethingsModified();
        }
    }
    
    /**
     * Remove all voxel filters.
     */
    public final synchronized void clearFilters() {
        if (scene != null) {
            scene.getVoxelSpaceComponent().resetFilters();
        }
    }
    
}
